use std::sync::Arc;

use serde::{Deserialize, Serialize};
use validation::Validation;

mod validation;

#[derive(Deserialize, Serialize, Clone, Debug)]
#[serde(rename_all = "camelCase")]
pub struct Parameter {
    name: Arc<str>,
    #[serde(rename = "type")]
    data_type: DataType,
    #[serde(skip)]
    is_array: bool,
    aliases: Vec<Arc<str>>,
    description: Option<Arc<str>>,
    help_message: Option<Arc<str>>,
    parameter_sets: Vec<ParameterSet>,
    validations: Vec<Validation>,
}

#[derive(Deserialize, Serialize, Clone, PartialEq, Eq, Default, Debug)]
pub struct ParameterSet {
    name: Arc<str>,
    mandatory: bool,
    position: Option<u8>,
    value_from_pipeline: Option<bool>,
    value_from_pipeline_by_property_name: Option<bool>,
}

#[derive(Deserialize, Serialize, Clone, Debug)]
pub enum DataType {
    DateTime,
    //TODO: Make this Variant obsolete by implementing De-/Serialize on the using Structs
    #[serde(rename = "String[]")]
    StringArray,
    String,
    Int32,
    Int64,
    #[serde(rename = "SwitchParameter")]
    Switch,
    Boolean,
    #[serde(rename = "PSCredential")]
    Credential,
    SecureString,
}
